@@ -649,14 +649,14 @@ class Exploit < Msf::Module
   # Returns true if the exploit has an aggressive stance.
   #
   def aggressive?
-    (stance == Stance::Aggressive || stance.include?(Stance::Aggressive))
+    (stance == Stance::Aggressive)
   end
 
   #
   # Returns if the exploit has a passive stance.
   #
   def passive?
-    (stance == Stance::Passive || stance.include?(Stance::Passive))
+    (stance == Stance::Passive)
   end
 
   #
